@@ -5,11 +5,10 @@ import zipfile
 from platform import architecture
 
 import click
-
 from picomc.account import AccountError
-from picomc.globals import am, gconf, platform, vm
+from picomc.env import Env, get_filepath
 from picomc.logging import logger
-from picomc.utils import ConfigLoader, get_filepath, join_classpath
+from picomc.utils import ConfigLoader, assert_java, join_classpath
 
 
 class NativesExtractor:
@@ -30,6 +29,7 @@ class NativesExtractor:
             dedup.add(fullpath)
             logger.debug("Extracting natives archive: {}".format(fullpath))
             with zipfile.ZipFile(fullpath) as zf:
+                # TODO take exclude into account
                 zf.extractall(path=self.ndir)
 
     def __exit__(self, ext_type, exc_value, traceback):
@@ -67,7 +67,7 @@ def process_arguments(arguments_dict):
             # and an "arch" rule may be present instead. Possible values are
             # currently unknown.
             if "name" in rule["os"]:
-                osmatch = osmatch and rule["os"]["name"] == platform
+                osmatch = osmatch and rule["os"]["name"] == Env.platform
             if "arch" in rule["os"]:
                 logger.warn("Matching arch rule, this may not work.")
                 arch = {"32": "x86"}.get(architecture()[0][:2], "?")
@@ -105,7 +105,7 @@ class BackupDict(dict):
         try:
             return dict.__getitem__(self, i)
         except KeyError:
-            return gconf[i]
+            return Env.gconf[i]
 
 
 class InstanceConfigLoader(ConfigLoader):
@@ -135,22 +135,26 @@ class Instance:
     def populate(self, version):
         self.config["version"] = version
 
-    def launch(self, account, version):
-        vobj = vm.get_version(version or self.config["version"])
+    def launch(self, account, version=None):
+        vobj = Env.vm.get_version(version or self.config["version"])
         logger.info("Launching instance {}!".format(self.name))
         logger.info("Using minecraft version: {}".format(vobj.version_name))
-        vobj.prepare()
         logger.info("Using account: {}".format(account))
-        os.makedirs(get_filepath("instances", self.name, "minecraft"), exist_ok=True)
+        gamedir = get_filepath("instances", self.name, "minecraft")
+        os.makedirs(gamedir, exist_ok=True)
+        vobj.prepare_launch(gamedir)
         with NativesExtractor(self, vobj):
-            self._exec_mc(account, vobj)
+            self._exec_mc(account, vobj, gamedir)
 
-    def _exec_mc(self, account, v):
+    def _exec_mc(self, account, v, gamedir):
         # this is temporary. FIXME
         # This 'function' is quickly getting worse and worse.
         # Rewrite it.
 
         java = [self.get_java()]
+
+        assert_java(java[0])
+
         java.append("-Xms{}".format(self.config["java.memory.min"]))
         java.append("-Xmx{}".format(self.config["java.memory.max"]))
         java += self.config["java.jvmargs"].split()
@@ -164,7 +168,6 @@ class Instance:
 
         # Make functions out of these two
         natives = get_filepath("instances", self.name, "natives")
-        gamedir = get_filepath("instances", self.name, "minecraft")
 
         mc = v.vspec.mainClass
 
@@ -203,8 +206,7 @@ class Instance:
                 game_directory=gamedir,
                 assets_root=get_filepath("assets"),
                 assets_index_name=v.vspec.assetIndex["id"],
-                # FIXME Ugly hack relying on untested behaviour:
-                game_assets=get_filepath("assets", "virtual", "legacy"),
+                game_assets=v.get_virtual_asset_path(),
             )
             smcargs.append(a)
 
@@ -230,6 +232,9 @@ def instance_list():
     )
 
 
+g_iname = ""
+
+
 @click.group()
 @click.argument("instance_name")
 def instance_cli(instance_name):
@@ -271,9 +276,9 @@ def remove():
 def launch(account, version_override):
     """Launch the instance."""
     if account is None:
-        account = am.get_default()
+        account = Env.am.get_default()
     else:
-        account = am.get(account)
+        account = Env.am.get(account)
     if not Instance.exists(g_iname):
         logger.error("No such instance exists.")
         return
