@@ -1,10 +1,13 @@
 import hashlib
 import json
 import os
+import subprocess
 import sys
 from functools import partial
+from os.path import expanduser, join
+from types import SimpleNamespace
 
-from picomc.globals import get_app_root
+from picomc.env import get_filepath
 from picomc.logging import logger
 
 
@@ -20,35 +23,10 @@ class cached_property(object):
         return r
 
 
-def get_filepath(*f):
-    return os.path.join(get_app_root(), *f)
-
-
 def join_classpath(*cp):
     return os.pathsep.join(cp)
 
 
-def check_directories():
-    """Create directory structure for the application."""
-    dirs = [
-        "",
-        "instances",
-        "versions",
-        "assets",
-        "assets/indexes",
-        "assets/objects",
-        "assets/virtual",
-        "libraries",
-    ]
-    for d in dirs:
-        path = get_filepath(*d.split("/"))
-        try:
-            os.makedirs(path)
-            logger.debug("Created dir: {}".format(path))
-        except FileExistsError:
-            pass
-
-
 def write_profiles_dummy():
     # This file makes the forge installer happy.
     fname = get_filepath("launcher_profiles.json")
@@ -56,6 +34,11 @@ def write_profiles_dummy():
         fd.write(r'{"profiles":{}}')
 
 
+def file_verify_relative(path, sha1):
+    abspath = get_filepath(path)
+    return os.path.isfile(abspath) and file_sha1(abspath) == sha1
+
+
 def file_sha1(filename):
     h = hashlib.sha1()
     with open(filename, "rb", buffering=0) as f:
@@ -69,6 +52,25 @@ def die(mesg, code=1):
     sys.exit(code)
 
 
+def assert_java(java):
+    try:
+        ret = subprocess.run([java, "-version"], capture_output=True)
+        version = ret.stderr.decode("utf8").splitlines()[0]
+        # This check is probably not bulletproof
+        logger.info("Using java version: {}".format(version))
+        if "1.8.0_" not in version:
+            logger.warn(
+                "Minecraft uses java 1.8.0 by default."
+                " You may experience issues, especially with older versions of Minecraft."
+            )
+    except FileNotFoundError:
+        die(
+            "Could not execute java at: {}. Have you installed it? Is it in yout PATH?".format(
+                java
+            )
+        )
+
+
 class ConfigLoader:
     def __init__(self, config_file, defaults={}, dict_impl=dict):
         self.filename = get_filepath(config_file)
