@@ -3,18 +3,20 @@ import operator
 import os
 import posixpath
 import re
+import shutil
 import sys
 import urllib.parse
 import urllib.request
+from collections import defaultdict
 from functools import reduce
 from platform import architecture
 
 import click
 import requests
 from picomc.downloader import DownloadQueue
-from picomc.globals import platform, vm
+from picomc.env import Env, get_filepath
 from picomc.logging import logger
-from picomc.utils import cached_property, die, file_sha1, get_filepath
+from picomc.utils import cached_property, die, file_sha1, file_verify_relative
 
 
 class VersionType:
@@ -138,7 +140,7 @@ class Version:
         fpath = get_filepath(
             "versions", self.version_name, "{}.json".format(self.version_name)
         )
-        ver = vm.get_manifest_version(self.version_name)
+        ver = Env.vm.get_manifest_version(self.version_name)
         if not ver:
             if os.path.exists(fpath):
                 logger.debug("Found custom vspec ({})".format(self.version_name))
@@ -206,7 +208,7 @@ class Version:
             for rule in rules:
                 osmatch = True
                 if "os" in rule:
-                    osmatch = rule["os"]["name"] == platform
+                    osmatch = rule["os"]["name"] == Env.platform
                 if osmatch:
                     allow = rule["action"] == "allow"
             if not allow:
@@ -221,21 +223,40 @@ class Version:
         # For some reason I don't remember, we are constructing the paths
         # to library downloads manually instead of using the url and hash
         # provided in the vspec. This should probably be reworked and hashes
-        # should be checked instead of just the filenames
+        # should be checked instead of just the filenames.
+        #
+        # The reason is that the downloads tag is only present in vanilla vspec,
+        # forge, optifine, fabric don't provide it.
         suffix = ""
+        sha = None
         if "natives" in lib:
-            if platform in lib["natives"]:
-                suffix = "-" + lib["natives"][platform]
+            if Env.platform in lib["natives"]:
+                suffix = "-" + lib["natives"][Env.platform]
                 # FIXME this is an ugly hack
                 suffix = suffix.replace("${arch}", architecture()[0][:2])
+                try:
+                    sha = lib["downloads"]["classifiers"]["natives-" + Env.platform][
+                        "sha1"
+                    ]
+                except KeyError:
+                    pass
             else:
                 logger.warn(
                     (
                         "Native library ({}) not available"
                         "for current platform ({}). Ignoring."
-                    ).format(lib["name"], platform)
+                    ).format(lib["name"], Env.platform)
                 )
                 return None
+        else:
+            try:
+                sha = lib["downloads"]["artifact"]["sha1"]
+            except KeyError:
+                pass
+
+        if sha is None:
+            logger.debug("Library {} has no sha in vspec".format(lib["name"]))
+
         fullname = lib["name"]
         url_base = lib.get("url", Version.LIBRARIES_URL)
         p, n, v, *va = fullname.split(":")
@@ -254,6 +275,8 @@ class Version:
             local_relpath = os.path.join(*fullpath.split("/"))
             local_abspath = os.path.join(basedir, local_relpath)
 
+        LibPaths.sha = sha
+
         return LibPaths
 
     def lib_filenames(self, natives=False):
@@ -294,59 +317,73 @@ class Version:
             paths = self._resolve_library(lib)
             if not paths:
                 continue
-            if force or not os.path.exists(paths.local_abspath):
+            ok = (
+                os.path.isfile(paths.local_abspath)
+                and os.path.getsize(paths.local_abspath) > 0
+            )
+            if paths.sha is not None:
+                ok = ok and file_sha1(paths.local_abspath) == paths.sha
+            if force or not ok:
                 q.add(paths.url, paths.local_relpath)
         q.download(paths.basedir)
 
+    def _populate_virtual_assets(self, where):
+        for name, obj in self.raw_asset_index["objects"].items():
+            sha = obj["hash"]
+            objpath = get_filepath("assets", "objects", sha[0:2], sha)
+            path = os.path.join(where, *name.split("/"))
+            # Maybe check file hash first? Would that be faster?
+            os.makedirs(os.path.dirname(path), exist_ok=True)
+            shutil.copy(get_filepath(objpath), os.path.join(where, path))
+
+    def get_virtual_asset_path(self):
+        return get_filepath("assets", "virtual", self.vspec.assetIndex["id"])
+
+    def prepare_assets_launch(self, gamedir):
+        is_map_resources = self.raw_asset_index.get("map_to_resources", False)
+        if is_map_resources:
+            logger.info("Mapping resources")
+            where = os.path.join(gamedir, "resources")
+            logger.debug("Resources path: {}".format(where))
+            self._populate_virtual_assets(where)
+
     def download_assets(self, force=False):
         """Downloads missing assets."""
-        # Produce reverse dict, as multiple files with a single hash
-        # can exist and should only be downloaded once.
-        rev = dict()
-        for name, obj in self.raw_asset_index["objects"].items():
-            h = obj["hash"]
-            if h in rev:
-                rev[h].append(name)
-            else:
-                rev[h] = [name]
 
-        logger.info("Checking {} assets.".format(len(rev.keys())))
+        hashes = set()
+        for obj in self.raw_asset_index["objects"].values():
+            hashes.add(obj["hash"])
+
+        logger.info("Checking {} assets.".format(len(hashes)))
 
         is_virtual = self.raw_asset_index.get("virtual", False)
 
         q = DownloadQueue()
-        for digest, names in rev.items():
-            # This is a mess, should be rewritten. FIXME
-            fname = os.path.join("objects", digest[0:2], digest)
-            vfnames = (
-                os.path.join("virtual", "legacy", *name.split("/")) for name in names
-            )
-            fullfname = get_filepath("assets", "objects", digest[0:2], digest)
-            url = urllib.parse.urljoin(
-                self.ASSETS_URL, "{}/{}".format(digest[0:2], digest)
-            )
-            outs = []
-            # FIXME: probably should be checking hashes here...
-            if force or not os.path.exists(fullfname):
-                outs.append(fname)
-            if is_virtual:
-                for vfname in vfnames:
-                    if force or not os.path.exists(
-                        get_filepath("assets", *vfname.split("/"))
-                    ):
-                        outs.append(vfname)
-            if outs:
-                q.add(url, *outs)
+        for sha in hashes:
+            path = os.path.join("assets", "objects", sha[0:2], sha)
+            if file_verify_relative(path, sha):
+                continue
+            url = urllib.parse.urljoin(self.ASSETS_URL, posixpath.join(sha[0:2], sha))
+            q.add(url, path)
 
         logger.info("Downloading {} assets. This could take a while.".format(len(q)))
+        q.download(Env.app_root)
 
-        q.download(get_filepath("assets"))
+        if is_virtual:
+            logger.info("Copying virtual assets")
+            where = self.get_virtual_asset_path()
+            logger.debug("Virtual asset path: {}".format(where))
+            self._populate_virtual_assets(where)
 
     def prepare(self):
         self.download_jarfile()
         self.download_libraries()
         self.download_assets()
 
+    def prepare_launch(self, gamedir):
+        self.prepare()
+        self.prepare_assets_launch(gamedir)
+
 
 class VersionManager:
     VERSION_MANIFEST_URL = (
@@ -413,7 +450,7 @@ g_vobj = None
 def version_cli(version_name):
     """Operate on local Minecraft versions."""
     global g_vobj
-    g_vobj = vm.get_version(version_name)
+    g_vobj = Env.vm.get_version(version_name)
 
 
 @click.command()
@@ -434,7 +471,7 @@ def list_versions(release, snapshot, alpha, beta, local, all):
         )
         local = True
     T = VersionType.create(release, snapshot, alpha, beta)
-    print("\n".join(vm.version_list(vtype=T, local=local)))
+    print("\n".join(Env.vm.version_list(vtype=T, local=local)))
 
 
 @version_cli.command()
